# Optum DOD - Evaluation and Management Extracts
# Individuals from 19 to 64 years old

library(arrow)
library(data.table)
library(fs)
library(plyr)
library(dplyr)
library(stringr)
library(xlsx)
library(haven)
library(zoo)
library("rio")


#-------- Exposed Cohort 2019-2021 --------#

rm(list=ls())
quart <- list("2019q1","2019q2","2019q3","2019q4","2020q1",
              "2020q2","2020q3","2020q4","2021q1")

cpt_codes = c("99202","99203","99204","99205",
              "99212","99213","99214","99215")

for (i in quart) {
  
  # Open medical procedure table
  assign(paste0("med",i), fread(paste0("/N/project/optum/data/raw/dodr/dod_m",i,".txt.gz"), sep="|", header =TRUE, select =c("PATID","PAT_PLANID","CLMID","CLMSEQ","LOC_CD","FST_DT","PROC_CD","PROV","PROVCAT")))
  # Keep CPT E/M codes
  assign(paste0("med",i), get(paste0("med",i))[PROC_CD %in% cpt_codes,])
  
  # Merge Medical procedure and Provider bridge
  assign(paste0("prov_bridge"), fread(paste0("/N/project/optum/data/raw/dodr/dod_provider_bridge",".txt.gz"), sep="|", header =TRUE, select =c("PROV","PROV_UNIQUE","NPI")))
  assign(paste0("med",i), merge(get(paste0("med",i)),prov_bridge, by=c("PROV"), all.x=TRUE))
  rm(prov_bridge)
  
  # Merge Medical procedure and Provider
  assign(paste0("provider"), fread(paste0("/N/project/optum/data/raw/dodr/dod_provider",".txt.gz"), sep="|", header =TRUE, select =c("PROV_UNIQUE","GRP_PRACTICE","HOSP_AFFIL","PROV_STATE","PROV_TYPE","TAXONOMY1","TAXONOMY2")))
  assign(paste0("med",i), merge(get(paste0("med",i)),provider, by=c("PROV_UNIQUE"), all.x=TRUE))
  rm(provider)
}

# Append datasets by quarter
med1921 <- rbind(med2019q1,med2019q2,med2019q3,med2019q4,med2020q1,
                 med2020q2,med2020q3,med2020q4,med2021q1)
rm(med2019q1,med2019q2,med2019q3,med2019q4,med2020q1,
   med2020q2,med2020q3,med2020q4,med2021q1)

## Member continuous enrollment
members_co <-fread("/N/project/optum/data/raw/dodr/dod_mbr_co_enroll_r.txt.gz", sep="|", header =TRUE, select =c("PATID","YRDOB","ELIGEFF","ELIGEND","GDR_CD"))
# Keep eligible patients by enrollment date
members_co <- subset(members_co,ELIGEFF<="2019-02-01" & ELIGEND>="2021-03-31",select=PATID:GDR_CD)
# Keep sample of adults between 19-64 years old
members_co <-subset(members_co,YRDOB>=1956 & YRDOB<=2001)

## Merge DOD data
dod_df <-fread("/N/project/optum/data/raw/dodr/dod_mbr_enroll_r.txt.gz", 
               sep="|", header =TRUE, select =c("PATID","RACE","STATE","ELIGEFF","FAMILY_ID"))
# Sort by PATID and ELIGEFF, then Keep latest State and family ID
dod_df <- dod_df[order(dod_df$PATID,-dod_df$ELIGEFF),]
dod_df <- dod_df[!duplicated(dod_df$PATID),]
dod_df <- subset(dod_df,select = -c(ELIGEFF))
members_co <-merge(members_co,dod_df, by="PATID")
export(members_co,"/N/project/optum_freedman/Extracts/DOD/cohort_adults_1921_exposed_dod.csv")

## Merge E&M and cohort data
merged1921 <-merge(med1921,members_co, by="PATID")

## Export file
export(merged1921,"/N/project/optum_freedman/Extracts/DOD/exposed_cpteval1921_adults_dod.csv")
rm(list=ls())


#-------- Unexposed Cohort 2018-2020 --------#

quart <-list("2018q1","2018q2","2018q3","2018q4","2019q1",
             "2019q2","2019q3","2019q4","2020q1")

cpt_codes = c("99212","99213","99214","99215",
              "99202","99203","99204","99205")

for (i in quart) {
  
  # Open medical procedure table
  assign(paste0("med",i), fread(paste0("/N/project/optum/data/raw/dodr/dod_m",i,".txt.gz"), sep="|", header =TRUE, select =c("PATID","PAT_PLANID","CLMID","CLMSEQ","LOC_CD","FST_DT","PROC_CD","PROV","PROVCAT")))
  # Keep CPT E/M codes
  assign(paste0("med",i), get(paste0("med",i))[PROC_CD %in% cpt_codes,])
  
  # Merge Medical procedure and Provider bridge
  assign(paste0("prov_bridge"), fread(paste0("/N/project/optum/data/raw/dodr/dod_provider_bridge",".txt.gz"), sep="|", header =TRUE, select =c("PROV","PROV_UNIQUE","NPI")))
  assign(paste0("med",i), merge(get(paste0("med",i)),prov_bridge, by=c("PROV"), all.x=TRUE))
  rm(prov_bridge)
  
  # Merge Medical procedure and Provider
  assign(paste0("provider"), fread(paste0("/N/project/optum/data/raw/dodr/dod_provider",".txt.gz"), sep="|", header =TRUE, select =c("PROV_UNIQUE","GRP_PRACTICE","HOSP_AFFIL","PROV_STATE","PROV_TYPE","TAXONOMY1","TAXONOMY2")))
  assign(paste0("med",i), merge(get(paste0("med",i)),provider, by=c("PROV_UNIQUE"), all.x=TRUE))
  rm(provider)
}

# Append datasets by quarter
med1820 <- rbind(med2018q1,med2018q2,med2018q3,med2018q4,med2019q1,
                 med2019q2,med2019q3,med2019q4,med2020q1)
rm(med2018q1,med2018q2,med2018q3,med2018q4,med2019q1,
   med2019q2,med2019q3,med2019q4,med2020q1)

## Member continuous enrollment
members_co <-fread("/N/project/optum/data/raw/dodr/dod_mbr_co_enroll_r.txt.gz", sep="|", header =TRUE, select =c("PATID","YRDOB","ELIGEFF","ELIGEND","GDR_CD"))
# Keep eligible patients by enrollment date
members_co <- subset(members_co,ELIGEFF<="2018-02-01" & ELIGEND>="2020-03-31",select=PATID:GDR_CD)
# Keep sample of adults between 19-64 years old
members_co <-subset(members_co,YRDOB>=1956 & YRDOB<=2001)

## Merge DOD data
dod_df <-fread("/N/project/optum/data/raw/dodr/dod_mbr_enroll_r.txt.gz", 
               sep="|", header =TRUE, select =c("PATID","RACE","STATE","ELIGEFF","FAMILY_ID"))
# Sort by PATID and ELIGEFF, then Keep latest State and family ID
dod_df <- dod_df[order(dod_df$PATID,-dod_df$ELIGEFF),]
dod_df <- dod_df[!duplicated(dod_df$PATID),]
dod_df <- subset(dod_df,select = -c(ELIGEFF))
members_co <-merge(members_co,dod_df, by="PATID")
export(members_co,"/N/project/optum_freedman/Extracts/DOD/cohort_adults_1820_unexposed_dod.csv")

## Merge E&M and cohort data
merged1820 <-merge(med1820,members_co, by="PATID")

## Export file
export(merged1820,"/N/project/optum_freedman/Extracts/DOD/unexposed_cpteval1820_adults_dod.csv")
rm(list=ls())